<?php

namespace Strig\Image\Sprite;

/**
 * Class Sprite
 *
 * @package Ep
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-core
 */
class Sprite
{
    const IMAGES_PER_ROW = 1000;

    /** @var string */
    protected $name;
    /** @var string */
    protected $baseDir;
    /** @var string */
    protected $outputDir;
    /** @var SpriteImage[] */
    protected $images = [];
    /** @var bool */
    protected $dirty = false;
    /** @var array */
    protected $sizes = [];

    /**
     * Sprite constructor.
     *
     * @param string $name
     * @param array $config
     */
    public function __construct(string $name, array $config)
    {
        $this->name = $name;
        $this->baseDir = $config['baseDir'];
        $this->outputDir = $config['outputDir'];

        if (!is_dir($this->outputDir)) {
            mkdir($this->outputDir, 0777, true);
        }

//        $this->outputDir = realpath($this->outputDir);

        $jsonPath = sprintf('%s/%s.json', $this->outputDir, $this->name);

        if (file_exists($jsonPath)) {
            $this->sizes = json_decode((string)file_get_contents($jsonPath), true);
        }
    }

    /**
     * Renderira sliku u sprajtu.
     *
     * @param string $imgName Ime slike
     * @param int $width Sirina
     * @param int $height Visina
     *
     * @return string
     * @throws \ImagickException
     */
    public function render($imgName, int $width, int $height, $key = null)
    {
        $size = sprintf('%dx%d', $width, $height);
        $key = $key ?? $imgName;

        if (!isset($this->sizes[$size])) {
            $this->sizes[$size] = [
                'name' => [],
                'map' => [],
            ];
        }

        if (!isset($this->sizes[$size]['name'][$key])) {
            $this->sizes[$size]['name'][$key] = $this->add($imgName, $width, $height);
        }

        [$indexX, $indexY] = $this->sizes[$size]['name'][$key];

        $bgLeft = $indexX * $width;
        $bgTop = $indexY * $height;

        return sprintf(
            '%d %d %d',
            $bgLeft,
            $bgTop,
            $indexX
        );
    }


    /**
     * Sprema promjene.
     * @throws \ImagickException
     */
    public function flush()
    {
        if ($this->dirty) {
            foreach ($this->images as $image) {
                $image->flush();
            }

            $jsonPath = sprintf('%s/%s.json', $this->outputDir, $this->name);
            $json = json_encode($this->sizes);

            file_put_contents($jsonPath, $json);
        }
    }

    /**
     * Dodaje sliku u sprite i vraca koordinate.
     *
     * @param string $imgName
     * @param int $width
     * @param int $height
     *
     * @return array
     * @throws \ImagickException
     */
    protected function add(string $imgName, int $width, int $height)
    {
        $image = $this->getImage($width, $height);
        $size = sprintf('%dx%d', $width, $height);
        $imgPath = 'https://img.carparts.hr/' . $imgName;

        // sliku dodajemo u zadnji red
        $indexY = !empty($this->sizes[$size]['map']) ? count($this->sizes[$size]['map']) - 1 : 0;
        // ako red ne postoji, napravimo ga
        if (!isset($this->sizes[$size]['map'][$indexY])) {
            $this->sizes[$size]['map'][$indexY] = [];
        }

        // ako vec imamo dovoljno slika u ovom redu, pravimo novi red
        if (count($this->sizes[$size]['map'][$indexY]) === self::IMAGES_PER_ROW) {
            $indexY++;
            $this->sizes[$size]['map'][$indexY] = [];
        }

        $indexX = count($this->sizes[$size]['map'][$indexY]);

        $image->add($imgPath, $width, $height, $indexX, $indexY);
        $this->sizes[$size]['map'][$indexY][$indexX] = $imgName;
        $this->dirty = true;

        return [$indexX, $indexY];
    }

    /**
     * @param int $width
     * @param int $height
     * @return string
     */
    public function getImageUrl(int $width, int $height)
    {
        return sprintf(
            '/%s/%s-%dx%d.png',
            $this->outputDir,
            $this->name,
            $width,
            $height
        );
    }

    /**
     * @return string
     */
    public function css()
    {
        $classes = [
            '.sprite{background-size: auto 100%;background-position: calc(-100% * var(--index)) 0;}',
        ];

        for ($i = 0; $i < 250; $i++) {
            $classes[] = '.sprite[data-index="' . $i . '"]{--index:' . $i . '}';
        }

        foreach (array_keys($this->sizes) as $size) {


            [$width, $height] = array_map('intval', explode('x', $size));
            $classes[] = sprintf(
                '.sprite-%s-%s{background-image:url(%s);width:%dpx;height:%dpx}',
                $this->name,
                $size,
                $this->getImageUrl($width, $height),
                $width,
                $height
            );
        }

        return implode($classes);
    }

    public function getIndex($key, $width, $height) {
        $size = sprintf('%dx%d', $width, $height);
        if (isset($this->sizes[$size]['name'][$key])) {
            $coordinates = $this->sizes[$size]['name'][$key];
            return $coordinates[0];
        }

        return 0;
    }

    public function cssCoordinates($key, $width, $height)
    {
        $size = sprintf('%dx%d', $width, $height);
        if (isset($this->sizes[$size]['name'][$key])) {
            $coordinates = $this->sizes[$size]['name'][$key];
            return sprintf('background-position: -%dpx -%dpx', $coordinates[0] * $width, $coordinates[1] * $height);
        }

        return '';
    }

    /**
     * Vraca sliku za date dimenzije.
     *
     * @param int $width
     * @param int $height
     *
     * @return SpriteImage
     */
    protected function getImage(int $width, int $height)
    {
        $fileName = sprintf('%s-%dx%d.png', $this->name, $width, $height);

        if (!isset($this->images[$fileName])) {
            $filePath = sprintf('%s/%s', $this->outputDir, $fileName);

            $image = new SpriteImage($this->baseDir, $filePath, $width, $height);

            $this->images[$fileName] = $image;
        }

        return $this->images[$fileName];
    }
}
