<?php

namespace Strig\Image\Sprite;

use Imagick;
use Strig\Image\Image;

/**
 * Class Image
 *
 * @package Ep
 * @author Elektronika PerhoÄ <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-core
 */
class SpriteImage
{
    /** @var string */
    protected $path;
    /** @var string */
    protected $output;
    /** @var int */
    protected $width;
    /** @var int */
    protected $height;
    /** @var bool */
    protected $dirty = false;

    protected $image;
    protected $images;

    /**
     * Image constructor.
     *
     * @param string $path Putanja do slike
     * @param string $output
     * @param int $width Å irina jedne slike
     * @param int $height Visina jedne slike
     */
    public function __construct(string $path, string $output, int $width, int $height)
    {
        $this->path = $path;
        $this->output = $output;
        $this->width = $width;
        $this->height = $height;
    }

    /**
     * Dodaje sprajt u sliku.
     *
     * @param string $imagePath
     * @param int $width
     * @param int $height
     * @param int $indexX
     * @param int $indexY
     * @throws \ImagickException
     */
    public function add(string $imagePath, int $width, int $height, int $indexX, int $indexY)
    {
        $image = $this->getImage($indexY);
        $source = new Imagick();
        $source->readImageBlob(file_get_contents($imagePath));
        $source->resizeImage($width, $height, 0, .75);

        $image->addImage($source);
        $this->dirty = true;
    }

    /**
     * Sprema sliku na disk.
     * @throws \ImagickException
     */
    public function flush()
    {
        if (!$this->dirty) {
            return;
        }

        $image = new Imagick();

        /** @var Imagick $row */
        foreach ($this->images as $row) {
            $row->resetIterator();
            $row = $row->appendImages(false);

            $image->addImage($row);
        }

        $image->resetIterator();

        $image = $image->appendImages(true);

        // reduce file size
        $image->setFormat('PNG');
        $image->setImageDepth(8);
        $image->stripImage();

        $image->writeImage($this->output);
        $this->compressPng($this->output);
    }

    /**
     * @param int $row
     * @return Imagick
     * @throws \ImagickException
     */
    protected function getImage(int $row)
    {
        if (!isset($this->images[$row])) {
            $this->images[$row] = new Imagick();
        }

        return $this->images[$row];
    }

    /**
     * Optimizes PNG file with pngquant 1.8 or later (reduces file size of 24-bit/32-bit PNG images).
     *
     * @param $pathToPngFile
     * @param int $maxQuality
     * @return string - content of PNG file after conversion
     */
    protected function compressPng($pathToPngFile, $maxQuality = 90)
    {
        $filename = escapeshellarg($pathToPngFile);
        $pathToCompressedPngFile = $pathToPngFile . 'compressed.png';
        $compressedFilename = escapeshellarg($pathToCompressedPngFile);

        // guarantee that quality won't be worse than that.
        $minQuality = 50;

        // '-' makes it use stdout, required to save to $compressed_png_content variable
        // '<' makes it read from the given file path
        // -f forces file overwrite
        // escapeshellarg() makes this safe to use with any path
        if ($cmd = $this->commandExists("pngquant")) {
            $this->shellExec($cmd . " --quality=$minQuality-$maxQuality -f -o " . $filename . " - < " . $filename);

            // PNGCrush
            // dont use pngcrush without pngquant :c
            //
            // creates new, crushed png file
            // rename when finish ( overwrite )
            if ($cmd = $this->commandExists("pngcrush")) {
                $this->shellExec($cmd . " -reduce -brute " . $filename . " " . $compressedFilename);
                rename($pathToCompressedPngFile, $pathToPngFile);
            }

        }


        return true;
    }

    protected function commandExists($cmd)
    {
        return $this->shellExec(sprintf("which %s", escapeshellarg($cmd)));
    }

    // stop writing to stdout/logs
    protected function shellExec($command, $stdout = false)
    {
        if (!$stdout) {
            $command .= ' 2> /dev/null';
        }

        return trim(shell_exec($command));
    }
}
